unsigned char* lpCodeStart;
int DebugSpare = 0;

struct ArgStruct
{
	int type;
	int val;
};

struct clsScriptContext
{
	clsChar* pCurChar;
	clsEquip* pCurEquip;
	int nCurFeat;
	string *pCurText, *pCurMainText, *pCurSubText;
	char* scriptName;

	ArgStruct Args[2048];
	int ArgCount;
};

int ParseScript(clsScriptContext* pContext, unsigned char* script, int depth = 0, char* pThisSymbol = "");

#include "scriptvar.h"
#include "scriptfuncs.h"
#include "scriptvalset.h"

int EvalExpr(clsScriptContext* pContext, unsigned char* script)
{
	int ArgOld, progress;
	int EvalVal;

	if (DebugOutput) {printf(" Evaluating:");}
	ArgOld = pContext->ArgCount;

	progress = ParseScript(pContext, script, 1);

	if (pContext->Args[ArgOld + 1].val == '+')
	{
		EvalVal = pContext->Args[ArgOld].val + pContext->Args[ArgOld + 2].val;
	}
	else if (pContext->Args[ArgOld + 1].val == '-')
	{
		EvalVal = pContext->Args[ArgOld].val - pContext->Args[ArgOld + 2].val;
	}
	else if (pContext->Args[ArgOld + 1].val == '*')
	{
		EvalVal = pContext->Args[ArgOld].val * pContext->Args[ArgOld + 2].val;
	}
	else if (pContext->Args[ArgOld + 1].val == '/')
	{
		EvalVal = pContext->Args[ArgOld].val / pContext->Args[ArgOld + 2].val;
	}
	else if (pContext->Args[ArgOld + 1].val == '%')
	{
		EvalVal = pContext->Args[ArgOld].val % pContext->Args[ArgOld + 2].val;
	}

	pContext->ArgCount = ArgOld;

	pContext->Args[pContext->ArgCount].type = 0;
	pContext->Args[pContext->ArgCount].val = EvalVal;
	pContext->ArgCount++;

	if (DebugOutput) {printf(" Evaluation Returned %d:", EvalVal);}

	return(progress);
}

inline int EvalMaxMin(clsScriptContext* pContext, unsigned char* script, int Max)
{
	int ArgOld, progress;
	int EvalVal;

	if (DebugOutput) {printf(" Max Function:");}

	ArgOld = pContext->ArgCount;

	progress = ParseScript(pContext, script, 1);

	if (Max)
	{
		EvalVal = max(pContext->Args[ArgOld].val, pContext->Args[ArgOld + 1].val);
	}
	else
	{
		EvalVal = min(pContext->Args[ArgOld].val, pContext->Args[ArgOld + 1].val);
	}

	pContext->ArgCount = ArgOld;
	pContext->Args[pContext->ArgCount].type = 0;
	pContext->Args[pContext->ArgCount].val = EvalVal;
	pContext->ArgCount++;

	if (DebugOutput) {printf(" Max/Min Returned %d:", EvalVal);}

	return(progress);
}

int EvalMax(clsScriptContext* pContext, unsigned char* script)
{
	return(EvalMaxMin(pContext, script, 1));
}

int EvalMin(clsScriptContext* pContext, unsigned char* script)
{
	return(EvalMaxMin(pContext, script, 0));
}

void LockCharToInt(clsScriptContext* pContext, int nArg)
{
	int converttemp;

	if (pContext->Args[nArg].type == 1)
	{
		converttemp = atoi((char*) pContext->Args[nArg].val);
		pContext->Args[nArg].val = converttemp;
		pContext->Args[nArg].type = 0;
	}
}

int condition_types(clsScriptContext* pContext, unsigned char* script)
{
	int progress;
	progress = ParseScript(pContext, script, 1);

	if(*script == 1 && *(script + 5) == 1 && strcmp(Source.lpSymbolStart + *((int*) (script + 6)), "condition_inverse") == 0)
	{
		pContext->Args[pContext->ArgCount - 1].val = !pContext->Args[pContext->ArgCount - 1].val;
	}

	return(progress);
}

int condition_all(clsScriptContext* pContext, unsigned char* script)
{
	int progress, ArgOld;
	
	ArgOld = pContext->ArgCount;
	progress = ParseScript(pContext, script, 1);

	if (pContext->ArgCount == ArgOld + 3)
	{
		if (pContext->Args[ArgOld + 1].val == '&')
		{
			pContext->Args[ArgOld].val = pContext->Args[ArgOld].val && pContext->Args[ArgOld + 2].val;
		}
		else if (pContext->Args[ArgOld + 1].val == '|')
		{
			pContext->Args[ArgOld].val = pContext->Args[ArgOld].val || pContext->Args[ArgOld + 2].val;
		}
		else
		{
			printf("Fatal Scripting Error\n");
			quitprog();
		}
		pContext->ArgCount = ArgOld + 1;
	}

	return(progress);
}

int condition(clsScriptContext* pContext, unsigned char* script)
{
	int progress, ArgOld;
	int Result;
	int comparetype;

	bool ConvertToInt = false;

	if (DebugOutput) {printf(" Checking condition:");}

	ArgOld = pContext->ArgCount;

	progress = ParseScript(pContext, script, 1);

	comparetype = TwoCharToInt((char*) pContext->Args[ArgOld + 1].val, true);

	if (comparetype != TwoCharToInt("==") && comparetype != TwoCharToInt("!="))
	{
		ConvertToInt = true;
	}
	if (pContext->Args[ArgOld].type == 0 || pContext->Args[ArgOld + 2].type == 0)
	{
		ConvertToInt = true;
	}

	if (ConvertToInt)
	{
		LockCharToInt(pContext, ArgOld);
		LockCharToInt(pContext, ArgOld + 2);
	}

	if (comparetype == TwoCharToInt("==") || comparetype == TwoCharToInt("!="))
	{
		if (pContext->Args[ArgOld].type == 0)
		{
			Result = (pContext->Args[ArgOld].val == pContext->Args[ArgOld + 2].val);
		}
		else
		{
			Result = (strcmp((char*) pContext->Args[ArgOld].val, (char*) pContext->Args[ArgOld + 2].val) == 0);
		}
		if (comparetype == TwoCharToInt("!="))
		{
			Result ^= 1;
		}
	}
	else if (comparetype == TwoCharToInt("<="))
	{
		Result = (pContext->Args[ArgOld].val <= pContext->Args[ArgOld + 2].val);
	}
	else if (comparetype == TwoCharToInt(">="))
	{
		Result = (pContext->Args[ArgOld].val >= pContext->Args[ArgOld + 2].val);
	}
	else if (comparetype == '<')
	{
		Result = (pContext->Args[ArgOld].val < pContext->Args[ArgOld + 2].val);
	}
	else if (comparetype == '>')
	{
		Result = (pContext->Args[ArgOld].val > pContext->Args[ArgOld + 2].val);
	}
	
	pContext->ArgCount = ArgOld;

	pContext->Args[pContext->ArgCount].type = 0;
	pContext->Args[pContext->ArgCount].val = Result;
	pContext->ArgCount++;
	
	return(progress);
}

int sIfDone[256];
int sIfApplied[256];
int sIfStack = -1;

int sIf(clsScriptContext* pContext, unsigned char* script)
{
	int ArgOld, progress;

	if (DebugOutput) {printf(" Doing If conditional Order:");}

	if (sIfStack == 255)
	{
		printf("Fatal Error\nIf Stack exceeded\n");
		quitprog();
	}

	sIfStack++;
	sIfDone[sIfStack] = 0;

	ArgOld = pContext->ArgCount;

	progress = ParseScript(pContext, script, 1);

	sIfStack--;

	pContext->ArgCount = ArgOld;

	if (DebugOutput) {printf(" If conditional Order ended:");}

	return(progress);
}

int sWhile(clsScriptContext* pContext, unsigned char* script)
{
	int progress, ArgOld;

	if (DebugOutput) {printf(" Doing While Order:");}

	if (sIfStack == 255)
	{
		printf("Fatal Error\nIf Stack exceeded\n");
		quitprog();
	}

	ArgOld = pContext->ArgCount;

	sIfStack++;
	sIfApplied[sIfStack] = 1;

	while (sIfApplied[sIfStack])
	{
		sIfApplied[sIfStack] = 0;
		sIfDone[sIfStack] = 0;
		progress = ParseScript(pContext, script, 1);
	}
	
	sIfStack--;
	pContext->ArgCount = ArgOld;

	if (DebugOutput) {printf(" While Order ended:");}

	return(progress);
}

int SkipScript(unsigned char* script, int num = 1)
{
	int i = 0;

	do
	{
		if (script[i] == 0)
		{
			num--;
			i++;
		}
		else if (script[i] == 1)
		{
			num++;
			i += 5;
		}
		else if (script[i] == 2)
		{
			i += 5;
		}
		else
		{
			while (script[i] >= 10)
			{
				i++;
			}
		}
	}
	while (num);

	return(i);
}

inline int sIfa(clsScriptContext* pContext, unsigned char* script, int condition)
{
	int ArgOld, progress, tmpProgress;
	int DoThis;

	ArgOld = pContext->ArgCount;

	if (DebugOutput) {printf(" If block found:");}

	if (sIfDone[sIfStack])
	{
		if (DebugOutput) {printf(" Skipping block:");}
		progress = SkipScript(script);
	}
	else
	{
		//progress = ParseScript(pContext, script);
		if (condition)
		{
			if (DebugOutput) {printf(" Checking condition:");}
			progress = ParseScript(pContext, script);
			DoThis = pContext->Args[pContext->ArgCount - 1].val;
		}
		else
		{
			if (DebugOutput) {printf(" No condition (Else block):");}
			DoThis = 1;
			progress = 0;
		}
		pContext->ArgCount = ArgOld;

		if (DoThis)
		{
			if (DebugOutput) {printf(" Performing block:");}
			sIfDone[sIfStack] = 1;
			tmpProgress = ParseScript(pContext, script + progress);
			if (tmpProgress == 1)
			{
				progress++;
			}
			else
			{
				progress += tmpProgress;
				progress += SkipScript(script + progress);
			}
			//progress += ParseScript(pContext, script + progress);
			
			sIfApplied[sIfStack] = 1;
		}
		else
		{
			if (DebugOutput) {printf(" Condition did not aply:");}
			progress += SkipScript(script + progress);
			sIfApplied[sIfStack] = 0;
		}
	}

	if (DebugOutput) {printf(" If block ended:");}

	pContext->ArgCount = ArgOld;
	return(progress);
}

int sIfa_i(clsScriptContext* pContext, unsigned char* script)
{
	return(sIfa(pContext, script, 1));
}

int sIfa_e(clsScriptContext* pContext, unsigned char* script)
{
	return(sIfa(pContext, script, 0));
}

int sFor(clsScriptContext* pContext, unsigned char* script)
{
	int progress, ArgOld;
	int pos_condition, pos_increment, pos_order;
	int loopok;

	ArgOld = pContext->ArgCount;

	//progress = ParseScript(pContext, script); //"for("
	progress = ParseScript(pContext, script); //valset
	//progress += ParseScript(pContext, script + progress); //";"

	pos_condition = progress;
	progress += SkipScript(script + progress, 0); //condition

	//progress += ParseScript(pContext, script + progress); //";"

	pos_increment = progress;
	progress += SkipScript(script + progress, 0); //increment

	//progress += ParseScript(pContext, script + progress); //"){"

	pos_order = progress;

	do
	{
		ParseScript(pContext, script + pos_condition);
		pContext->ArgCount--;
		if (pContext->Args[pContext->ArgCount].val == 1)
		{
			progress = ParseScript(pContext, script + pos_order);
			ParseScript(pContext, script + pos_increment);
			loopok = 1;
		}
		else
		{
			progress += pos_order + (progress == 1 ? 0 : SkipScript(script + pos_order + progress));
			loopok = 0;
		}
	} while (loopok);

	pContext->ArgCount = ArgOld;
	return(progress);
}

int SetEquip(clsScriptContext* pContext, unsigned char* script)
{
	int i;
	for (i = 0;i < pContext->pCurChar->EquipCount;i++)
	{
		if (pContext->pCurChar->Equipment[i].ID == pContext->pCurChar->FeatDependancy[pContext->nCurFeat])
		{
			pContext->pCurEquip = &pContext->pCurChar->Equipment[i];
			goto Found;
		}
	}
	pContext->pCurEquip = &eDumpEquip;
Found:
	return(ParseScript(pContext, script, 1));
}

int DoAllEq(clsScriptContext* pContext, unsigned char* script)
{
	int i, progress;
	clsEquip* EquipOld;
	EquipOld = pContext->pCurEquip;

	if (pContext->pCurChar->EquipCount)
	{
		for (i = 0;i < pContext->pCurChar->EquipCount;i++)
		{
			pContext->pCurEquip = &pContext->pCurChar->Equipment[i];
			progress = ParseScript(pContext, script, 1);
		}
	}
	else
	{
		progress = SkipScript(script, 1);
	}

	pContext->pCurEquip = EquipOld;
	return(progress);
}

#include "scriptextract.h"
#include "scriptredir.h"

int ParseScript(clsScriptContext* pContext, unsigned char* script, int depth, char* pThisSymbol)
{
	int i;
	char* pSymbol;
	char DebugTemp[8];

	DebugSpare++;

	i = 0;
	do
	{
		if (script[i] == 0)
		{
			i++;
			depth--;
		}
		else if (script[i] == 1)
		{
			i++;
			pSymbol = (char*) Source.lpSymbolStart + *((int*) &script[i]);
			i += sizeof(int);

			if (DebugOutput)
			{
				sprintf(DebugTemp, "%%%ds", DebugSpare * 2);
				printf("\n");
				printf(DebugTemp, "");
				printf("%4d - %s:", (&script[i] - lpCodeStart - 5), pSymbol);
			}

			if (*((int*) (RedirFuncs + *((int*) &script[i - sizeof(int)]))))
			{
				i += ((int(*)(clsScriptContext*, unsigned char*)) *((int*) (RedirFuncs + *((int*) &script[i - sizeof(int)]))))(pContext, &script[i]);

			}
			else
			{
				i += ParseScript(pContext, &script[i], 1, pSymbol);
			}
		}
		else if (script[i] == 2)
		{
			i += 5;
			if (DebugOutput)
			{
				printf("%d", *(int*)(&script[i-4]));
			}
		}
		else
		{
			char CharOut[2] = " ";
			while (script[i] >= 10)
			{
				if (DebugOutput)
				{
					CharOut[0] = script[i];
					printf("%s", CharOut);
				}
				i++;
			}
		}
	} while (depth > 0);

	if (*pThisSymbol && *((int*) (RedirExtract + (pThisSymbol - Source.lpSymbolStart))))
	{
		((int(*)(clsScriptContext*, int)) *((int*) (RedirExtract + (pThisSymbol - Source.lpSymbolStart))))(pContext, (int) script);
	}

	DebugSpare--;

	return(i);
}

int RunScript(clsChar* TargetStruct, clsEquip* TargetStructE, string* TextOutput, char* script, const char* pScriptName, int Feat = -1, string* SubText = NULL)
{
	lpCodeStart = (unsigned char*) script;

	clsScriptContext Context;

	Context.pCurChar = TargetStruct;
	Context.pCurEquip = TargetStructE;
	Context.pCurText = TextOutput;
	Context.pCurMainText = TextOutput;
	Context.pCurSubText = SubText;
	Context.nCurFeat = Feat;
	Context.scriptName = (char*) pScriptName;
	Context.ArgCount = 0;

	return(ParseScript(&Context, lpCodeStart));
}
